// ====================================================================
// This file is part of FlexibleSUSY.
//
// FlexibleSUSY is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published
// by the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// FlexibleSUSY is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with FlexibleSUSY.  If not, see
// <http://www.gnu.org/licenses/>.
// ====================================================================

// File generated at Sun 26 Aug 2018 13:50:13

#ifndef MSSMNoFVatMGUTHimalaya_EFFECTIVE_COUPLINGS_H
#define MSSMNoFVatMGUTHimalaya_EFFECTIVE_COUPLINGS_H

#include "MSSMNoFVatMGUTHimalaya_mass_eigenstates.hpp"
#include "lowe.h"
#include "physical_input.hpp"
#include "standard_model.hpp"

#include <complex>
#include <Eigen/Core>

namespace flexiblesusy {

namespace standard_model {
class Standard_model;
}

class MSSMNoFVatMGUTHimalaya_effective_couplings {
public:
   MSSMNoFVatMGUTHimalaya_effective_couplings(const MSSMNoFVatMGUTHimalaya_mass_eigenstates&,
                                   const softsusy::QedQcd&,
                                   const Physical_input&);

   void do_run_couplings(bool flag) { rg_improve = flag; }
   bool do_run_couplings() const { return rg_improve; }
   void do_include_qcd_corrections(bool flag) { include_qcd_corrections = flag; }
   bool do_include_qcd_corrections() const { return include_qcd_corrections; }
   void set_physical_inputs(const Physical_input& inputs_) { physical_input = inputs_; }
   void set_low_energy_data(const softsusy::QedQcd& qedqcd_) { qedqcd = qedqcd_; }
   void set_model(const MSSMNoFVatMGUTHimalaya_mass_eigenstates& model_);

   double get_hhVPVP_partial_width(int gO1) const;
   double get_hhVGVG_partial_width(int gO1) const;
   double get_AhVPVP_partial_width(int gO1) const;
   double get_AhVGVG_partial_width(int gO1) const;
   std::complex<double> get_eff_CphhVPVP(int gO1) const { return eff_CphhVPVP(gO1); }
   std::complex<double> get_eff_CphhVGVG(int gO1) const { return eff_CphhVGVG(gO1); }
   std::complex<double> get_eff_CpAhVPVP(int gO1) const { return eff_CpAhVPVP(gO1); }
   std::complex<double> get_eff_CpAhVGVG(int gO1) const { return eff_CpAhVGVG(gO1); }

   void calculate_effective_couplings();

   std::complex<double> CphhconjVWmVWm(int gI2) const;
   std::complex<double> CpbarFdFdhhPL(int gI1) const;
   std::complex<double> CpbarFdFdAhPL(int gI2) const;
   std::complex<double> CpbarFsFshhPL(int gI1) const;
   std::complex<double> CpbarFsFsAhPL(int gI2) const;
   std::complex<double> CpbarFbFbhhPL(int gI1) const;
   std::complex<double> CpbarFbFbAhPL(int gI2) const;
   std::complex<double> CpbarFuFuhhPL(int gI1) const;
   std::complex<double> CpbarFuFuAhPL(int gI2) const;
   std::complex<double> CpbarFcFchhPL(int gI1) const;
   std::complex<double> CpbarFcFcAhPL(int gI2) const;
   std::complex<double> CpbarFtFthhPL(int gI1) const;
   std::complex<double> CpbarFtFtAhPL(int gI2) const;
   std::complex<double> CpbarFeFehhPL(int gI1) const;
   std::complex<double> CpbarFeFeAhPL(int gI2) const;
   std::complex<double> CpbarFmFmhhPL(int gI1) const;
   std::complex<double> CpbarFmFmAhPL(int gI2) const;
   std::complex<double> CpbarFtauFtauhhPL(int gI1) const;
   std::complex<double> CpbarFtauFtauAhPL(int gI2) const;
   std::complex<double> CphhSdconjSd(int gt1, int gt2, int gt3) const;
   std::complex<double> CphhSuconjSu(int gt1, int gt2, int gt3) const;
   std::complex<double> CphhSeconjSe(int gt1, int gt2, int gt3) const;
   std::complex<double> CphhSmconjSm(int gt1, int gt2, int gt3) const;
   std::complex<double> CphhStauconjStau(int gt1, int gt2, int gt3) const;
   std::complex<double> CphhSsconjSs(int gt1, int gt2, int gt3) const;
   std::complex<double> CphhScconjSc(int gt1, int gt2, int gt3) const;
   std::complex<double> CphhSbconjSb(int gt1, int gt2, int gt3) const;
   std::complex<double> CphhStconjSt(int gt1, int gt2, int gt3) const;
   std::complex<double> CphhHpmconjHpm(int gt1, int gt2, int gt3) const;
   std::complex<double> CpbarChaChahhPL(int gt3, int gt1, int gt2) const;
   std::complex<double> CpAhSdconjSd(int gt1, int gt2, int gt3) const;
   std::complex<double> CpAhSuconjSu(int gt1, int gt2, int gt3) const;
   std::complex<double> CpAhSeconjSe(int gt1, int gt2, int gt3) const;
   std::complex<double> CpAhSmconjSm(int gt1, int gt2, int gt3) const;
   std::complex<double> CpAhStauconjStau(int gt1, int gt2, int gt3) const;
   std::complex<double> CpAhSsconjSs(int gt1, int gt2, int gt3) const;
   std::complex<double> CpAhScconjSc(int gt1, int gt2, int gt3) const;
   std::complex<double> CpAhSbconjSb(int gt1, int gt2, int gt3) const;
   std::complex<double> CpAhStconjSt(int gt1, int gt2, int gt3) const;
   std::complex<double> CpAhHpmconjHpm(int gt1, int gt2, int gt3) const;
   std::complex<double> CpbarChaChaAhPL(int gt3, int gt2, int gt1) const;
   void calculate_eff_CphhVPVP(int gO1);
   void calculate_eff_CphhVGVG(int gO1);
   void calculate_eff_CpAhVPVP(int gO1);
   void calculate_eff_CpAhVGVG(int gO1);

private:
   MSSMNoFVatMGUTHimalaya_mass_eigenstates model;
   softsusy::QedQcd qedqcd;
   Physical_input physical_input;
   bool rg_improve;
   bool include_qcd_corrections;

   void copy_mixing_matrices_from_model();

   standard_model::Standard_model initialise_SM() const;
   void run_SM_strong_coupling_to(standard_model::Standard_model, double m);

   // higher order corrections to the amplitudes for
   // effective coupling to photons
   std::complex<double> scalar_scalar_qcd_factor(double, double) const;
   std::complex<double> scalar_fermion_qcd_factor(double, double) const;
   std::complex<double> pseudoscalar_fermion_qcd_factor(double, double) const;

   // higher order corrections to the leading order
   // effective couplings to gluons
   double number_of_active_flavours(double) const;
   double scalar_scaling_factor(double) const;
   double pseudoscalar_scaling_factor(double) const;

   Eigen::Matrix<double,2,2> ZD{Eigen::Matrix<double,2,2>::Zero()};
   Eigen::Matrix<double,2,2> ZU{Eigen::Matrix<double,2,2>::Zero()};
   Eigen::Matrix<double,2,2> ZE{Eigen::Matrix<double,2,2>::Zero()};
   Eigen::Matrix<double,2,2> ZM{Eigen::Matrix<double,2,2>::Zero()};
   Eigen::Matrix<double,2,2> ZTau{Eigen::Matrix<double,2,2>::Zero()};
   Eigen::Matrix<double,2,2> ZS{Eigen::Matrix<double,2,2>::Zero()};
   Eigen::Matrix<double,2,2> ZC{Eigen::Matrix<double,2,2>::Zero()};
   Eigen::Matrix<double,2,2> ZB{Eigen::Matrix<double,2,2>::Zero()};
   Eigen::Matrix<double,2,2> ZT{Eigen::Matrix<double,2,2>::Zero()};
   Eigen::Matrix<double,2,2> ZH{Eigen::Matrix<double,2,2>::Zero()};
   Eigen::Matrix<double,2,2> ZA{Eigen::Matrix<double,2,2>::Zero()};
   Eigen::Matrix<double,2,2> ZP{Eigen::Matrix<double,2,2>::Zero()};
   Eigen::Matrix<std::complex<double>,4,4> ZN{Eigen::Matrix<std::complex<double>,4,4>::Zero()};
   Eigen::Matrix<std::complex<double>,2,2> UM{Eigen::Matrix<std::complex<double>,2,2>::Zero()};
   Eigen::Matrix<std::complex<double>,2,2> UP{Eigen::Matrix<std::complex<double>,2,2>::Zero()};
   Eigen::Matrix<double,2,2> ZZ{Eigen::Matrix<double,2,2>::Zero()};

   Eigen::Array<std::complex<double>,2,1> eff_CphhVPVP;
   Eigen::Array<std::complex<double>,2,1> eff_CphhVGVG;
   Eigen::Array<std::complex<double>,2,1> eff_CpAhVPVP;
   Eigen::Array<std::complex<double>,2,1> eff_CpAhVGVG;

};

} // namespace flexiblesusy

#endif
